=== added directory 'cmd/qtcontact-test'
=== added file 'cmd/qtcontact-test/main.go'
--- cmd/qtcontact-test/main.go	1970-01-01 00:00:00 +0000
+++ cmd/qtcontact-test/main.go	2014-07-30 19:30:22 +0000
@@ -0,0 +1,18 @@
+package main
+
+import (
+	"fmt"
+	"os"
+
+	"launchpad.net/account-polld/qtcontact"
+)
+
+func main() {
+	if len(os.Args) != 2 {
+		fmt.Println("usage:", os.Args[0], "[email address]")
+		os.Exit(1)
+	}
+
+	path := qtcontact.GetAvatar(os.Args[1])
+	fmt.Println("Avatar found:", path)
+}

=== modified file 'plugins/gmail/gmail.go'
--- plugins/gmail/gmail.go	2014-07-29 11:45:57 +0000
+++ plugins/gmail/gmail.go	2014-07-30 19:30:22 +0000
@@ -21,12 +21,14 @@
 	"encoding/json"
 	"fmt"
 	"net/http"
+	"net/mail"
 	"net/url"
 	"os"
 	"sort"
 
 	"launchpad.net/account-polld/accounts"
 	"launchpad.net/account-polld/plugins"
+	"launchpad.net/account-polld/qtcontact"
 )
 
 const (
@@ -93,9 +95,16 @@
 		if _, ok := pushMsgMap[msg.ThreadId]; ok {
 			pushMsgMap[msg.ThreadId].Notification.Card.Summary += fmt.Sprintf(", %s", hdr[hdrFROM])
 		} else {
-			summary := fmt.Sprintf("Message \"%s\" from %s", hdr[hdrSUBJECT], hdr[hdrFROM])
+			var from, avatar string
+			from = hdr[hdrFROM]
+			emailAddress, err := mail.ParseAddress(hdr[hdrFROM])
+			if err == nil {
+				from = emailAddress.Name
+				avatar = qtcontact.GetAvatar(emailAddress.Address)
+			}
+			summary := fmt.Sprintf("Message \"%s\" from %s", hdr[hdrSUBJECT], from)
 			action := "https://mail.google.com/mail/u/0/?pli=1#inbox/" + msg.ThreadId
-			pushMsgMap[msg.ThreadId] = *plugins.NewStandardPushMessage(summary, msg.Snippet, action, "")
+			pushMsgMap[msg.ThreadId] = *plugins.NewStandardPushMessage(summary, msg.Snippet, action, avatar)
 		}
 	}
 	var pushMsg []plugins.PushMessage

=== modified file 'qtcontact/contacts.go'
--- qtcontact/contacts.go	2014-07-29 18:31:45 +0000
+++ qtcontact/contacts.go	2014-07-30 19:30:22 +0000
@@ -1,4 +1,4 @@
-package main
+package qtcontact
 
 // #cgo CXXFLAGS: -std=c++0x -pedantic-errors -Wall -fno-strict-aliasing -I/usr/share/c++/4.8
 // #cgo LDFLAGS: -lstdc++
@@ -6,14 +6,40 @@
 // #include "qtcontacts.h"
 import "C"
 
-import "fmt"
-
-//export AvatarPath
-func AvatarPath(path *C.char) {
-	fmt.Println("email", C.GoString(path))
+import (
+	"sync"
+	"time"
+)
+
+var (
+	avatarPathChan chan string
+	m              sync.Mutex
+)
+
+//export callback
+func callback(path *C.char) {
+	avatarPathChan <- C.GoString(path)
 }
 
-func main() {
-	C.getAvatar(C.CString("sergiusens@gmail.com"))
-	C.getAvatar(C.CString("sergiusens@gmail.com"))
+// GetAvatar retrieves an avatar path for the specified email
+// address. Multiple calls to this func will be in sync
+func GetAvatar(emailAddress string) string {
+	if emailAddress == "" {
+		return ""
+	}
+
+	m.Lock()
+	defer m.Unlock()
+
+	avatarPathChan = make(chan string, 1)
+	defer close(avatarPathChan)
+
+	C.getAvatar(C.CString(emailAddress))
+
+	select {
+	case <-time.After(10 * time.Second):
+		return ""
+	case path := <-avatarPathChan:
+		return path
+	}
 }

=== modified file 'qtcontact/qtcontacts.cpp'
--- qtcontact/qtcontacts.cpp	2014-07-29 20:58:58 +0000
+++ qtcontact/qtcontacts.cpp	2014-07-30 19:30:22 +0000
@@ -22,21 +22,21 @@
 QTCONTACTS_USE_NAMESPACE
 
 void getAvatar(char *email) {
-    QScopedPointer<Avatar> avatar(new Avatar());
-    avatar->getThumbnail(email);
-}
-
-void Avatar::getThumbnail(char *email) {
     static char empty[1] = {0};
     static char *argv[] = {empty, empty, empty};
     static int argc = 1;
     QCoreApplication mApp(argc, argv);
 
+    QScopedPointer<Avatar> avatar(new Avatar());
+    avatar->getThumbnail(email);
+}
+
+void Avatar::getThumbnail(char *email) {
     // set the map for this object and the passed email
     _signalMapper->setMapping(this, QString(email));
 
     QTimer::singleShot(0, this, SLOT(emitSignals()));
-    mApp.exec();
+    QCoreApplication::instance()->exec();
 }
 
 
@@ -57,6 +57,7 @@
     QByteArray byteArray = avatar.toUtf8();
     char* cString = byteArray.data();
 
-    AvatarPath(cString);
+    callback(cString);
+    QCoreApplication::instance()->quit();
 }
 

